#!/usr/bin/python
#
#	@section COPYRIGHT
#	Copyright (C) 2019 Consequential Robotics Ltd
#	
#	@section AUTHOR
#	Consequential Robotics http://consequentialrobotics.com
#	
#	@section LICENSE
#	For a full copy of the license agreement, and a complete
#	definition of "The Software", see LICENSE in the MDK root
#	directory.
#	
#	Subject to the terms of this Agreement, Consequential
#	Robotics grants to you a limited, non-exclusive, non-
#	transferable license, without right to sub-license, to use
#	"The Software" in accordance with this Agreement and any
#	other written agreement with Consequential Robotics.
#	Consequential Robotics does not transfer the title of "The
#	Software" to you; the license granted to you is not a sale.
#	This agreement is a binding legal agreement between
#	Consequential Robotics and the purchasers or users of "The
#	Software".
#	
#	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
#	KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
#	WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
#	PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
#	OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
#	OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
#	OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#	This is a MIRO ROS client template for Python.

import rospy
import std_msgs
import geometry_msgs

import math
import numpy as np
import time
import sys
import os

import miro2 as miro

################################################################

def error(msg):

	print(msg)
	sys.exit(0)

################################################################

class controller:

	def callback_package(self, msg):

		# ignore until active
		if not self.active:
			return
			
		# store
		self.package = msg

	def loop(self):

		# output
		msg_cmd_vel = geometry_msgs.msg.TwistStamped()

		# desired wheel speed (m/sec)
		wheel_speed = [0.0, 0.0]

		# loop
		while self.active and not rospy.core.is_shutdown():

			# if sensor data has arrived
			if not self.package is None:

				# get light levels from sensor package (normalised in [0, 1])
				light = self.package.light.data

				# clear sensor package that has now been used
				self.package = None

				# convert to wheel speed (m/sec)
				k = 0.4
				wheel_speed[0] = k * light[0]
				wheel_speed[1] = k * light[1]
				
				# report
				l = int(math.ceil(light[0] * 20))
				r = int(math.ceil(light[1] * 20))
				print ' '*(20-l)+'L'+'='*l, '='*r+'R'+' '*(20-r), "\r",
				sys.stdout.flush()

				# convert wheel speed to command velocity (m/sec, Rad/sec)
				(dr, dtheta) = miro.utils.wheel_speed2cmd_vel(wheel_speed)

				# update message to publish to control/cmd_vel
				msg_cmd_vel.twist.linear.x = dr
				msg_cmd_vel.twist.angular.z = dtheta

				# publish message to topic
				self.pub_cmd_vel.publish(msg_cmd_vel)

			# yield
			time.sleep(0.01)
			self.t_now = self.t_now + 0.01
			
		# finalise report
		print "\n\nexit..."
		print " "

	def __init__(self, args):

		rospy.init_node("client_template", anonymous=True)

		# state
		self.active = False
		self.t_now = 0.0

		# arguments
		self.no_cliff = False

		# inputs
		self.package = None

		# handle args
		for arg in args:
			f = arg.find('=')
			if f == -1:
				key = arg
				val = ""
			else:
				key = arg[:f]
				val = arg[f+1:]
			if key == "--no-cliff":
				self.no_cliff = True
			else:
				error("argument not recognised \"" + arg + "\"")

		# robot name
		topic_base_name = "/" + os.getenv("MIRO_ROBOT_NAME")

		# publish
		topic = topic_base_name + "/control/flags"
		print ("publish", topic)
		self.pub_flags = rospy.Publisher(topic, std_msgs.msg.UInt32, queue_size=0)

		# publish
		topic = topic_base_name + "/control/cmd_vel"
		print ("publish", topic)
		self.pub_cmd_vel = rospy.Publisher(topic, geometry_msgs.msg.TwistStamped, queue_size=0)

		# subscribe
		topic = topic_base_name + "/sensors/package"
		print ("subscribe", topic)
		self.sub_package = rospy.Subscriber(topic, miro.msg.sensors_package, self.callback_package, queue_size=1, tcp_nodelay=True)

		# report
		print """
This example implements a Braitenberg vehicle type 2a.
https://en.wikipedia.org/wiki/Braitenberg_vehicle"""
		if self.no_cliff:
			print """
**** NB: You have disabled the cliff reflex - do not
let your MIRO approach a cliff!"""

		# wait for connect
		print "\nwait for connect... ",
		sys.stdout.flush()
		time.sleep(1)
		print "OK"
		
		# send control flags
		print "send control flags... ",
		msg = std_msgs.msg.UInt32()
		msg.data = 0
		if self.no_cliff:
			msg.data |= miro.constants.PLATFORM_D_FLAG_DISABLE_CLIFF_REFLEX
		self.pub_flags.publish(msg)
		print "OK"
		
		# set up for graph
		print "press CTRL+C to exit."
		print "graph below shows light level on left/right sensors:"
		print " "

		# set to active
		self.active = True

if __name__ == "__main__":

	# normal singular invocation
	main = controller(sys.argv[1:])
	main.loop()




